@@ -1,3 +1,4 @@
+ # vim: set sw=4 expandtab :
  #
  # Copyright 2004 Apache Software Foundation
  #
@@ -15,7 +16,7 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: apache.py 374268 2006-02-02 05:31:45Z nlehuen $
+ # $Id: apache.py 468216 2006-10-27 00:54:12Z grahamd $
 
 import sys
 import traceback
@@ -25,11 +26,21 @@ import pdb
 import stat
 import imp
 import types
+import cgi
 import _apache
 
-# a small hack to improve PythonPath performance. This
-# variable stores the last PythonPath in raw (unevaled) form.
-_path = None
+try:
+    import threading
+except:
+    import dummy_threading as threading
+
+# Cache for values of PythonPath that have been seen already.
+_path_cache = {}
+_path_cache_lock = threading.Lock()
+
+_result_warning = """Handler has returned result or raised SERVER_RETURN
+exception with argument having non integer type. Type of value returned
+was %s, whereas expected """ + str(types.IntType) + "."
 
 class CallBack:
     """
@@ -74,20 +85,19 @@ class CallBack:
             else:
                 object_str = l[1]
 
-            # add the directory to pythonpath if
-            # not there yet, or pythonpath specified
+            # evaluate pythonpath and set sys.path to
+            # resulting value if not already done
 
-            if config.has_key("PythonPath"):
-                # we want to do as little evaling as possible,
-                # so we remember the path in un-evaled form and
-                # compare it
-                global _path
-                pathstring = config["PythonPath"]
-                if pathstring != _path:
-                    _path = pathstring
-                    newpath = eval(pathstring)
-                    if sys.path != newpath:
-                        sys.path[:] = newpath
+            if config.has_key("PythonPath"): 
+                _path_cache_lock.acquire() 
+                try: 
+                    pathstring = config["PythonPath"] 
+                    if not _path_cache.has_key(pathstring): 
+                        newpath = eval(pathstring) 
+                        _path_cache[pathstring] = None 
+                        sys.path[:] = newpath 
+                finally: 
+                    _path_cache_lock.release()
 
             # import module
             module = import_module(module_name,
@@ -97,16 +107,37 @@ class CallBack:
             object = resolve_object(module, object_str,
                                     arg=conn, silent=0)
 
-            if object:
+            # Only permit debugging using pdb if Apache has
+            # actually been started in single process mode.
 
-                # call the object
-                if config.has_key("PythonEnablePdb"):
-                    result = pdb.runcall(object, conn)
-                else:
+            pdb_debug = int(config.get("PythonEnablePdb", "0"))
+            one_process = exists_config_define("ONE_PROCESS")
+
+            if pdb_debug and one_process:
+
+                # Don't use pdb.runcall() as it results in
+                # a bogus 'None' response when pdb session
+                # is quit. With this code the exception
+                # marking that the session has been quit is
+                # propogated back up and it is obvious in
+                # the error message what actually occurred.
+
+                debugger = pdb.Pdb()
+                debugger.reset()
+                sys.settrace(debugger.trace_dispatch)
+
+                try:
                     result = object(conn)
 
-                assert (type(result) == type(int())), \
-                       "ConnectionHandler '%s' returned invalid return code." % handler
+                finally:
+                    debugger.quitting = 1
+                    sys.settrace(None)
+
+            else:
+                result = object(conn)
+
+            assert (type(result) == type(int())), \
+                   "ConnectionHandler '%s' returned invalid return code." % handler
 
         except PROG_TRACEBACK, traceblock:
             # Program run-time error
@@ -140,55 +171,82 @@ class CallBack:
 
         try:
 
-            # split module::handler
-            l = filter.handler.split('::', 1)
-            module_name = l[0]
-            if len(l) == 1:
-                # no oject, provide default
-                if filter.is_input:
-                    object_str = "inputfilter"
-                else:
-                    object_str = "outputfilter"
-            else:
-                object_str = l[1]
-
             # add the directory to pythonpath if
-            # not there yet, or pythonpath specified
+            # not there yet, or evaluate pythonpath
+            # and set sys.path to resulting value
+            # if not already done
 
-            if config.has_key("PythonPath"):
-                # we want to do as little evaling as possible,
-                # so we remember the path in un-evaled form and
-                # compare it
-                global _path
-                pathstring = config["PythonPath"]
-                if pathstring != _path:
-                    _path = pathstring
-                    newpath = eval(pathstring)
-                    if sys.path != newpath:
-                        sys.path[:] = newpath
+            if config.has_key("PythonPath"): 
+                _path_cache_lock.acquire() 
+                try: 
+                    pathstring = config["PythonPath"] 
+                    if not _path_cache.has_key(pathstring): 
+                        newpath = eval(pathstring) 
+                        _path_cache[pathstring] = None 
+                        sys.path[:] = newpath 
+                finally: 
+                    _path_cache_lock.release()
             else:
                 if filter.dir and (filter.dir not in sys.path):
                     sys.path[:0] = [filter.dir]
 
-            # import module
-            module = import_module(module_name,
-                                   autoreload=int(config.get("PythonAutoReload", 1)),
-                                   log=debug)
+            if not callable(filter.handler):
 
-            # find the object
-            object = resolve_object(module, object_str, arg=filter, silent=0)
-
-            if object:
-
-                # call the object
-                if config.has_key("PythonEnablePdb"):
-                    pdb.runcall(object, filter)
+                # split module::handler
+                l = filter.handler.split('::', 1)
+                module_name = l[0]
+                if len(l) == 1:
+                    # no oject, provide default
+                    if filter.is_input:
+                        object_str = "inputfilter"
+                    else:
+                        object_str = "outputfilter"
                 else:
-                    object(filter)
+                    object_str = l[1]
 
-                # always flush the filter. without a FLUSH or EOS bucket,
-                # the content is never written to the network.
-                # XXX an alternative is to tell the user to flush() always
+                # import module
+                module = import_module(module_name,
+                                       autoreload=int(config.get("PythonAutoReload", 1)),
+                                       log=debug)
+
+                # find the object
+                object = resolve_object(module, object_str, arg=filter, silent=0)
+            else:
+                object = filter.handler
+
+            # Only permit debugging using pdb if Apache has
+            # actually been started in single process mode.
+
+            pdb_debug = int(config.get("PythonEnablePdb", "0"))
+            one_process = exists_config_define("ONE_PROCESS")
+
+            if pdb_debug and one_process:
+
+                # Don't use pdb.runcall() as it results in
+                # a bogus 'None' response when pdb session
+                # is quit. With this code the exception
+                # marking that the session has been quit is
+                # propogated back up and it is obvious in
+                # the error message what actually occurred.
+
+                debugger = pdb.Pdb()
+                debugger.reset()
+                sys.settrace(debugger.trace_dispatch)
+
+                try:
+                    result = object(filter)
+
+                finally:
+                    debugger.quitting = 1
+                    sys.settrace(None)
+
+            else:
+                result = object(filter)
+
+            # always flush the filter. without a FLUSH or EOS bucket,
+            # the content is never written to the network.
+            # XXX an alternative is to tell the user to flush() always
+            if not filter.closed: 
                 filter.flush()
 
         except SERVER_RETURN, value:
@@ -248,91 +306,125 @@ class CallBack:
         config = req.get_config()
         debug = int(config.get("PythonDebug", 0))
 
+        default_object_str = req.phase[len("python"):].lower()
+
+        # Lookup expected status values that allow us to
+        # continue when multiple handlers exist.
+
+        expected = _status_values[default_object_str]
+
         try:
             hlist = req.hlist
 
             while hlist.handler is not None:
 
-                # split module::handler
-                l = hlist.handler.split('::', 1)
-
-                module_name = l[0]
-                if len(l) == 1:
-                    # no oject, provide default
-                    object_str = req.phase[len("python"):].lower()
-                else:
-                    object_str = l[1]
-
                 # add the directory to pythonpath if
-                # not there yet, or pythonpath specified
-                if config.has_key("PythonPath"):
-                    # we want to do as little evaling as possible,
-                    # so we remember the path in un-evaled form and
-                    # compare it
-                    global _path
-                    pathstring = config["PythonPath"]
-                    if pathstring != _path:
-                        _path = pathstring
-                        newpath = eval(pathstring)
-                        if sys.path != newpath:
-                            sys.path[:] = newpath
+                # not there yet, or evaluate pythonpath
+                # and set sys.path to resulting value
+                # if not already done
+
+                if config.has_key("PythonPath"): 
+                    _path_cache_lock.acquire() 
+                    try: 
+                        pathstring = config["PythonPath"] 
+                        if not _path_cache.has_key(pathstring): 
+                            newpath = eval(pathstring) 
+                            _path_cache[pathstring] = None 
+                            sys.path[:] = newpath 
+                    finally: 
+                        _path_cache_lock.release()
                 else:
                     dir = hlist.directory
                     if dir and (dir not in sys.path):
                         sys.path[:0] = [dir]
 
-                # import module
-                module = import_module(module_name,
-                                       autoreload=int(config.get("PythonAutoReload", 1)),
-                                       log=debug)
+                if not callable(hlist.handler):
 
-                # find the object
-                object = resolve_object(module, object_str,
-                                        arg=req, silent=hlist.silent)
+                    # split module::handler
+                    l = hlist.handler.split('::', 1)
 
-                if object:
-
-                    # call the object
-                    if config.has_key("PythonEnablePdb"):
-                        result = pdb.runcall(object, req)
+                    module_name = l[0]
+                    if len(l) == 1:
+                        # no object, provide default
+                        object_str = default_object_str
                     else:
-                        result = object(req)
+                        object_str = l[1]
 
-                    assert (type(result) == type(int())), \
-                           "Handler '%s' returned invalid return code." % hlist.handler
+                    # import module
+                    module = import_module(module_name,
+                                           autoreload=int(config.get("PythonAutoReload", 1)),
+                                           log=debug)
+
+                    # find the object
+                    object = resolve_object(module, object_str,
+                                            arg=req, silent=hlist.silent)
+
+                else:
+                    object = hlist.handler
+
+                if not hlist.silent or object is not None:
+
+                    try:
+                        # Only permit debugging using pdb if Apache has
+                        # actually been started in single process mode.
+
+                        pdb_debug = int(config.get("PythonEnablePdb", "0"))
+                        one_process = exists_config_define("ONE_PROCESS")
+
+                        if pdb_debug and one_process:
+
+                            # Don't use pdb.runcall() as it results in
+                            # a bogus 'None' response when pdb session
+                            # is quit. With this code the exception
+                            # marking that the session has been quit is
+                            # propogated back up and it is obvious in
+                            # the error message what actually occurred.
+
+                            debugger = pdb.Pdb()
+                            debugger.reset()
+                            sys.settrace(debugger.trace_dispatch)
+
+                            try:
+                                result = object(req)
+
+                            finally:
+                                debugger.quitting = 1
+                                sys.settrace(None)
+
+                        else:
+                            result = object(req)
+
+                    except SERVER_RETURN, value:
+
+                        # The SERVER_RETURN exception type when raised
+                        # otherwise indicates an abort from below with
+                        # value as (result, status) or (result, None) or
+                        # result.
+
+                        if len(value.args) == 2:
+                            (result, status) = value.args
+                            if status:
+                                req.status = status
+                        else:
+                            result = value.args[0]
+
+                    assert (type(result) == types.IntType), \
+                            _result_warning % type(result)
 
                     # stop cycling through handlers
-                    if result != OK:
+                    if result not in expected:
                         break
 
                 elif hlist.silent:
-                    # A faulty handler marked as silent will only 
-                    # propagate DECLINED if it is the first and only handler.
-                    if result != OK:
+                    # A missing handler when in silent mode will
+                    # only propagate DECLINED if it is the first
+                    # and only handler.
+
+                    if result == HTTP_INTERNAL_SERVER_ERROR:
                         result = DECLINED
 
                 hlist.next()
 
-        except SERVER_RETURN, value:
-            # SERVER_RETURN indicates an abort from below
-            # with value as (result, status) or (result, None) or result
-            try:
-                if len(value.args) == 2:
-                    (result, status) = value.args
-                    if status:
-                        req.status = status
-                else:
-                    result = value.args[0]
-
-                if type(result) != type(7):
-                    s = "Value raised with SERVER_RETURN is invalid. It is a "
-                    s = s + "%s, but it must be a tuple or an int." % type(result)
-                    _apache.log_error(s, APLOG_NOERRNO|APLOG_ERR, req.server)
-                    return HTTP_INTERNAL_SERVER_ERROR
-
-            except:
-                pass
-
         except PROG_TRACEBACK, traceblock:
             # Program run-time error
             try:
@@ -354,6 +446,77 @@ class CallBack:
 
         return result
 
+    def IncludeDispatch(self, filter, tag, code):
+
+        try:
+            config = filter.req.get_config()
+            debug = int(config.get("PythonDebug", 0))
+
+            if not hasattr(filter.req,"ssi_globals"):
+                filter.req.ssi_globals = {}
+
+            filter.req.ssi_globals["filter"] = filter
+
+            filter.req.ssi_globals["__file__"] = filter.req.filename
+
+            code = code.replace('\r\n', '\n').rstrip()
+
+            if tag == 'eval':
+                result = eval(code, filter.req.ssi_globals)
+                if result is not None:
+                    filter.write(str(result))
+            elif tag == 'exec':
+                exec(code, filter.req.ssi_globals)
+
+            filter.flush()
+
+        except:
+            try:
+                exc_type, exc_value, exc_traceback = sys.exc_info()
+                result = self.ReportError(exc_type, exc_value, exc_traceback,
+                                          filter=filter, phase=filter.name,
+                                          hname=filter.req.filename,
+                                          debug=debug)
+            finally:
+                exc_traceback = None
+
+            raise
+
+        filter.req.ssi_globals["filter"] = None
+
+        return OK
+
+    def ImportDispatch(self, name):
+
+        config = main_server.get_config()
+
+        debug = int(config.get("PythonDebug", "0"))
+
+        # evaluate pythonpath and set sys.path to
+        # resulting value if not already done
+
+        if config.has_key("PythonPath"): 
+            _path_cache_lock.acquire() 
+            try: 
+                pathstring = config["PythonPath"] 
+                if not _path_cache.has_key(pathstring): 
+                    newpath = eval(pathstring) 
+                    _path_cache[pathstring] = None 
+                    sys.path[:] = newpath 
+            finally: 
+                _path_cache_lock.release()
+
+        # split module::function
+        l = name.split('::', 1)
+        module_name = l[0]
+        func_name = None
+        if len(l) != 1:
+            func_name = l[1]
+
+        module = import_module(module_name, log=debug)
+
+        if func_name:
+            getattr(module, func_name)()
 
     def ReportError(self, etype, evalue, etb, req=None, filter=None, srv=None,
                     phase="N/A", hname="N/A", debug=0):
@@ -384,11 +547,12 @@ class CallBack:
                     return HTTP_INTERNAL_SERVER_ERROR
                 else:
                     # write to client
+                    req.status = HTTP_INTERNAL_SERVER_ERROR
                     req.content_type = 'text/html'
 
                     s = '\n<pre>\nMod_python error: "%s %s"\n\n' % (phase, hname)
                     for e in traceback.format_exception(etype, evalue, etb):
-                        s = s + e + '\n'
+                        s = s + cgi.escape(e) + '\n'
                     s = s + "</pre>\n"
 
                     if filter:
@@ -401,6 +565,7 @@ class CallBack:
             except:
                 # last try
                 traceback.print_exc()
+                sys.stderr.flush()
 
         finally:
             # erase the traceback
@@ -484,20 +649,20 @@ def module_mtime(module):
     mtime = 0
     if module.__dict__.has_key("__file__"):
 
-       filepath = module.__file__
+        filepath = module.__file__
 
-       try:
-           # this try/except block is a workaround for a Python bug in
-           # 2.0, 2.1 and 2.1.1. See
-           # http://sourceforge.net/tracker/?group_id=5470&atid=105470&func=detail&aid=422004
+        try:
+            # this try/except block is a workaround for a Python bug in
+            # 2.0, 2.1 and 2.1.1. See
+            # http://sourceforge.net/tracker/?group_id=5470&atid=105470&func=detail&aid=422004
 
-           if os.path.exists(filepath):
-               mtime = os.path.getmtime(filepath)
+            if os.path.exists(filepath):
+                mtime = os.path.getmtime(filepath)
 
-           if os.path.exists(filepath[:-1]) :
-               mtime = max(mtime, os.path.getmtime(filepath[:-1]))
+            if os.path.exists(filepath[:-1]) :
+                mtime = max(mtime, os.path.getmtime(filepath[:-1]))
 
-       except OSError: pass
+        except OSError: pass
 
     return mtime
 
@@ -738,27 +903,39 @@ def restore_nocgi(sav_env, si, so):
     sys.stdout = si
     sys.stdin = so
 
-_interpreter = None
-_server = None
+interpreter = None
+main_server = None
 _callback = None
 
-def register_cleanup(handler,data=None):
-    _apache.register_cleanup(_interpreter,_server,handler,data)
+def register_cleanup(callback, data=None):
+    _apache.register_cleanup(interpreter, main_server, callback, data)
 
-def init(name,server):
+def init(name, server):
     """
         This function is called by the server at startup time
     """
 
-    global _interpreter
-    global _server
-    _interpreter = name
-    _server = server
+    global interpreter
+    global main_server
+    interpreter = name
+    main_server = server
 
     sys.argv = ["mod_python"]
 
     global _callback
     _callback = CallBack()
+
+    options = main_server.get_options()
+    #interpreters = options.get('mod_python.future.importer', None)
+    interpreters = options.get('mod_python.legacy.importer', None)
+    if interpreters:
+        interpreters = map(lambda x: x.strip(), interpreters.split(','))
+        #if '*' in interpreters or interpreter in interpreters:
+        if not ('*' in interpreters or interpreter in interpreters):
+            from mod_python import importer
+    else:
+        from mod_python import importer
+
     return _callback
 
 ## Some functions made public
@@ -769,6 +946,7 @@ config_tree = _apache.config_tree
 server_root = _apache.server_root
 mpm_query = _apache.mpm_query
 exists_config_define = _apache.exists_config_define
+stat = _apache.stat
 
 ## Some constants
 
@@ -811,6 +989,7 @@ HTTP_EXPECTATION_FAILED           = 417
 HTTP_UNPROCESSABLE_ENTITY         = 422
 HTTP_LOCKED                       = 423
 HTTP_FAILED_DEPENDENCY            = 424
+HTTP_UPGRADE_REQUIRED             = 426
 HTTP_INTERNAL_SERVER_ERROR        = 500
 HTTP_NOT_IMPLEMENTED              = 501
 HTTP_BAD_GATEWAY                  = 502
@@ -850,6 +1029,21 @@ OK = REQ_PROCEED = 0
 DONE = -2
 DECLINED = REQ_NOACTION = -1
 
+_status_values = {
+    "postreadrequesthandler":   [ DECLINED, OK ],
+    "transhandler":             [ DECLINED ],
+    "maptostoragehandler":      [ DECLINED ],
+    "inithandler":              [ DECLINED, OK ],
+    "headerparserhandler":      [ DECLINED, OK ],
+    "accesshandler":            [ DECLINED, OK ],
+    "authenhandler":            [ DECLINED ],
+    "authzhandler":             [ DECLINED ],
+    "typehandler":              [ DECLINED ],
+    "fixuphandler":             [ DECLINED, OK ],
+    "loghandler":               [ DECLINED, OK ],
+    "handler":                  [ OK ],
+}
+
 # constants for get_remote_host
 REMOTE_HOST = 0
 REMOTE_NAME = 1
@@ -875,7 +1069,7 @@ FINFO_MTIME = 8
 FINFO_CTIME = 9
 FINFO_FNAME = 10
 FINFO_NAME = 11
-#FINFO_FILEHAND = 14
+FINFO_FILETYPE = 12
 
 # the req.parsed_uri
 URI_SCHEME = 0
@@ -890,8 +1084,9 @@ URI_FRAGMENT = 8
 
 # for req.proxyreq
 PROXYREQ_NONE = 0       # No proxy
-PROXYREQ_PROXY = 1    # Standard proxy
+PROXYREQ_PROXY = 1      # Standard proxy
 PROXYREQ_REVERSE = 2    # Reverse proxy
+PROXYREQ_RESPONSE = 3   # Origin response
 
 # methods for req.allow_method()
 M_GET = 0               # RFC 2616: HTTP
@@ -952,3 +1147,54 @@ AP_MPMQ_MAX_SPARE_THREADS  = 10 # Max # of spare threads
 AP_MPMQ_MAX_REQUESTS_DAEMON= 11 # Max # of requests per daemon
 AP_MPMQ_MAX_DAEMONS        = 12 # Max # of daemons by config
 
+# magic mime types
+CGI_MAGIC_TYPE = "application/x-httpd-cgi" 
+INCLUDES_MAGIC_TYPE = "text/x-server-parsed-html" 
+INCLUDES_MAGIC_TYPE3 = "text/x-server-parsed-html3" 
+DIR_MAGIC_TYPE = "httpd/unix-directory" 
+
+# for req.read_body
+REQUEST_NO_BODY = 0 
+REQUEST_CHUNKED_ERROR = 1 
+REQUEST_CHUNKED_DECHUNK = 2 
+
+# for req.connection.keepalive
+AP_CONN_UNKNOWN = _apache.AP_CONN_UNKNOWN
+AP_CONN_CLOSE = _apache.AP_CONN_CLOSE
+AP_CONN_KEEPALIVE = _apache.AP_CONN_KEEPALIVE
+
+# for req.finfo[apache.FINFO_FILETYPE]
+APR_NOFILE = _apache.APR_NOFILE
+APR_REG = _apache.APR_REG
+APR_DIR = _apache.APR_DIR
+APR_CHR = _apache.APR_CHR
+APR_BLK = _apache.APR_BLK
+APR_PIPE = _apache.APR_PIPE
+APR_LNK = _apache.APR_LNK
+APR_SOCK = _apache.APR_SOCK
+APR_UNKFILE = _apache.APR_UNKFILE
+
+# for apache.stat()
+APR_FINFO_LINK = 0x00000001 # Stat the link not the file itself if it is a link
+APR_FINFO_MTIME = 0x00000010 # Modification Time
+APR_FINFO_CTIME = 0x00000020 # Creation or inode-changed time
+APR_FINFO_ATIME = 0x00000040 # Access Time
+APR_FINFO_SIZE = 0x00000100 # Size of the file
+APR_FINFO_CSIZE = 0x00000200 # Storage size consumed by the file
+APR_FINFO_DEV = 0x00001000 # Device
+APR_FINFO_INODE = 0x00002000 # Inode
+APR_FINFO_NLINK = 0x00004000 # Number of links
+APR_FINFO_TYPE = 0x00008000 # Type
+APR_FINFO_USER = 0x00010000 # User
+APR_FINFO_GROUP = 0x00020000 # Group
+APR_FINFO_UPROT = 0x00100000 # User protection bits
+APR_FINFO_GPROT = 0x00200000 # Group protection bits
+APR_FINFO_WPROT = 0x00400000 # World protection bits
+APR_FINFO_ICASE = 0x01000000 # if dev is case insensitive
+APR_FINFO_NAME = 0x02000000 # ->name in proper case
+APR_FINFO_MIN = 0x00008170 # type, mtime, ctime, atime, size
+APR_FINFO_IDENT = 0x00003000 # dev and inode
+APR_FINFO_OWNER = 0x00030000 # user and group
+APR_FINFO_PROT = 0x00700000 # all protections
+APR_FINFO_NORM = 0x0073b170 # an atomic unix apr_stat()
+APR_FINFO_DIRENT = 0x02000000 # an atomic unix apr_dir_read()
