@@ -15,7 +15,7 @@
  #
  # Originally developed by Gregory Trubetskoy.
  #
- # $Id: publisher.py 374295 2006-02-02 07:59:26Z nlehuen $
+ # $Id: publisher.py 468839 2006-10-29 07:07:21Z grahamd $
 
 """
   This handler is conceptually similar to Zope's ZPublisher, except
@@ -213,7 +213,7 @@ def handler(req):
     published = publish_object(req, object)
     
     # we log a message if nothing was published, it helps with debugging
-    if (not published) and (req.bytes_sent==0) and (req.next is None):
+    if (not published) and (req._bytes_queued==0) and (req.next is None):
         log=int(req.get_config().get("PythonDebug", 0))
         if log:
             req.log_error("mod_python.publisher: nothing to publish.")
@@ -224,48 +224,45 @@ def process_auth(req, object, realm="unknown", user=None, passwd=None):
 
     found_auth, found_access = 0, 0
 
-    # because ap_get_basic insists on making sure that AuthName and
-    # AuthType directives are specified and refuses to do anything
-    # otherwise (which is technically speaking a good thing), we
-    # have to do base64 decoding ourselves.
-    #
-    # to avoid needless header parsing, user and password are parsed
-    # once and the are received as arguments
-    if not user and req.headers_in.has_key("Authorization"):
-        try:
-            s = req.headers_in["Authorization"][6:]
-            s = base64.decodestring(s)
-            user, passwd = s.split(":", 1)
-        except:
-            raise apache.SERVER_RETURN, apache.HTTP_BAD_REQUEST
-
     if hasattr(object, "__auth_realm__"):
         realm = object.__auth_realm__
 
+    func_object = None
+
     if type(object) is FunctionType:
+        func_object = object
+    elif type(object) == types.MethodType:
+        func_object = object.im_func
+
+    if func_object:
         # functions are a bit tricky
 
-        if hasattr(object, "func_code"):
-            func_code = object.func_code
+        func_code = func_object.func_code
+        func_globals = func_object.func_globals
 
-            if "__auth__" in func_code.co_names:
-                i = list(func_code.co_names).index("__auth__")
-                __auth__ = func_code.co_consts[i+1]
-                if hasattr(__auth__, "co_name"):
-                    __auth__ = new.function(__auth__, globals())
-                found_auth = 1
+        def lookup(name):
+            i = None
+            if name in func_code.co_names:
+                i = list(func_code.co_names).index(name)
+            elif func_code.co_argcount < len(func_code.co_varnames):
+                names = func_code.co_varnames[func_code.co_argcount:]
+                if name in names:
+                    i = list(names).index(name)
+            if i is not None:
+                return (1, func_code.co_consts[i+1])
+            return (0, None)
 
-            if "__access__" in func_code.co_names:
-                # first check the constant names
-                i = list(func_code.co_names).index("__access__")
-                __access__ = func_code.co_consts[i+1]
-                if hasattr(__access__, "co_name"):
-                    __access__ = new.function(__access__, globals())
-                found_access = 1
+        (found_auth, __auth__) = lookup('__auth__')
+        if found_auth and type(__auth__) == types.CodeType:
+            __auth__ = new.function(__auth__, func_globals)
 
-            if "__auth_realm__" in func_code.co_names:
-                i = list(func_code.co_names).index("__auth_realm__")
-                realm = func_code.co_consts[i+1]
+        (found_access, __access__) = lookup('__access__')
+        if found_access and type(__access__) == types.CodeType:
+            __access__ = new.function(__access__, func_globals)
+
+        (found_realm, __auth_realm__) = lookup('__auth_realm__')
+        if found_realm:
+            realm = __auth_realm__
 
     else:
         if hasattr(object, "__auth__"):
@@ -275,6 +272,22 @@ def process_auth(req, object, realm="unknown", user=None, passwd=None):
             __access__ = object.__access__
             found_access = 1
 
+    if found_auth or found_access:
+        # because ap_get_basic insists on making sure that AuthName and
+        # AuthType directives are specified and refuses to do anything
+        # otherwise (which is technically speaking a good thing), we
+        # have to do base64 decoding ourselves.
+        #
+        # to avoid needless header parsing, user and password are parsed
+        # once and the are received as arguments
+        if not user and req.headers_in.has_key("Authorization"):
+            try:
+                s = req.headers_in["Authorization"][6:]
+                s = base64.decodestring(s)
+                user, passwd = s.split(":", 1)
+            except:
+                raise apache.SERVER_RETURN, apache.HTTP_BAD_REQUEST
+
     if found_auth:
 
         if not user:
@@ -464,11 +477,14 @@ def publish_object(req, object):
             if charset is not None:
                 req.content_type += '; charset=%s'%charset
         
-        # Write result even if req.method == 'HEAD'
-        # Apache will truncate the output if necessary.
-        # Truncating output here if req.method is HEAD is likely the 
-        # wrong thing to do # as it may cause problems for any output filters.
-        # See MODPYTHON-105 for details.
-        req.write(result)
+        # Write result even if req.method is 'HEAD' as Apache
+        # will discard the final output anyway. Truncating
+        # output here if req.method is 'HEAD' is likely the
+        # wrong thing to do as it may cause problems for any
+        # output filters. See MODPYTHON-105 for details. We
+        # also do not flush output as that will prohibit use
+        # of 'CONTENT_LENGTH' filter to add 'Content-Length'
+        # header automatically. See MODPYTHON-107 for details.
+        req.write(result, 0)
 
         return True
