@@ -18,7 +18,7 @@
  *
  * _apachemodule.c 
  *
- * $Id: _apachemodule.c 420297 2006-07-09 13:53:06Z nlehuen $
+ * $Id: _apachemodule.c 478403 2006-11-23 00:08:36Z grahamd $
  *
  */
 
@@ -89,6 +89,7 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
         return NULL; /* error */
 
     /* split query string by '&' and ';' into a list of pairs */
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */ 
     pairs = PyList_New(0);
     if (pairs == NULL)
         return NULL;
@@ -102,6 +103,7 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
         char *cpair;
         int j = 0;
 
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         pair = PyString_FromStringAndSize(NULL, len);
         if (pair == NULL)
             return NULL;
@@ -116,6 +118,7 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
         }
 
         if (j) {
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */ 
             _PyString_Resize(&pair, j);
             if (pair)
                 PyList_Append(pairs, pair);
@@ -134,6 +137,7 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
     if (dict == NULL)
         return NULL;
 
+    /* PYTHON 2.5: 'PyList_Size' uses Py_ssize_t for input parameters */ 
     lsize = PyList_Size(pairs);
     n = 0;
 
@@ -147,9 +151,11 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
         cpair = PyString_AS_STRING(pair);
 
         len = strlen(cpair);
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         key = PyString_FromStringAndSize(NULL, len);
         if (key == NULL) 
             return NULL;
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         val = PyString_FromStringAndSize(NULL, len);
         if (val == NULL) 
             return NULL;
@@ -184,7 +190,9 @@ static PyObject *parse_qs(PyObject *self, PyObject *args)
             ap_unescape_url(ckey);
             ap_unescape_url(cval);
 
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */ 
             _PyString_Resize(&key, strlen(ckey));
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */ 
             _PyString_Resize(&val, strlen(cval));
 
             if (key && val) {
@@ -237,6 +245,7 @@ static PyObject *parse_qsl(PyObject *self, PyObject *args)
         return NULL; /* error */
 
     /* split query string by '&' and ';' into a list of pairs */
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */ 
     pairs = PyList_New(0);
     if (pairs == NULL)
         return NULL;
@@ -250,6 +259,7 @@ static PyObject *parse_qsl(PyObject *self, PyObject *args)
         char *cpair, *ckey, *cval;
         int plen, j, p, k, v;
 
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         pair = PyString_FromStringAndSize(NULL, len);
         if (pair == NULL)
             return NULL;
@@ -271,14 +281,17 @@ static PyObject *parse_qsl(PyObject *self, PyObject *args)
         }
 
         cpair[j] = '\0';
+        /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */ 
         _PyString_Resize(&pair, j);
         cpair = PyString_AS_STRING(pair);
 
         /* split the "abc=def" pair */
         plen = strlen(cpair);
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         key = PyString_FromStringAndSize(NULL, plen);
         if (key == NULL) 
             return NULL;
+        /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
         val = PyString_FromStringAndSize(NULL, plen);
         if (val == NULL) 
             return NULL;
@@ -312,7 +325,9 @@ static PyObject *parse_qsl(PyObject *self, PyObject *args)
             ap_unescape_url(ckey);
             ap_unescape_url(cval);
 
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */
             _PyString_Resize(&key, strlen(ckey));
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */
             _PyString_Resize(&val, strlen(cval));
 
             if (key && val) {
@@ -486,21 +501,27 @@ static PyObject *_global_trylock(PyObject *self, PyObject *args)
         index = (hash % (glb->nlocks-1)+1);
     }
 
-//      ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
-//                "_global_trylock at index %d from pid %d", index, getpid());
+    /*  
+     * ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
+     *           "_global_trylock at index %d from pid %d", index, getpid());
+     */
     Py_BEGIN_ALLOW_THREADS
     rv = apr_global_mutex_trylock(glb->g_locks[index]);        
     Py_END_ALLOW_THREADS
     
     if (rv == APR_SUCCESS) {
-//      ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
-//                "_global_trylock DONE at index %d from pid %d", index, getpid());
+        /*
+         * ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
+         *         "_global_trylock DONE at index %d from pid %d", index, getpid());
+         */
         Py_INCREF(Py_True);
         return Py_True;
     }
     else if(APR_STATUS_IS_EBUSY(rv)) {
-//      ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
-//                "_global_trylock BUSY at index %d from pid %d", index, getpid());
+        /*
+         * ap_log_error(APLOG_MARK, APLOG_WARNING, 0, s,
+         *         "_global_trylock BUSY at index %d from pid %d", index, getpid());
+         */
         Py_INCREF(Py_False);
         return Py_False;
     }
@@ -682,6 +703,42 @@ static PyObject *exists_config_define(PyObject *self, PyObject *args)
     }
 }
 
+/**
+ ** mp_stat(fname, wanted)
+ **
+ *  Wrapper for apr_stat().
+ */
+
+static PyObject *mp_stat(PyObject *self, PyObject *args)
+{
+    char *fname = NULL;
+    apr_int32_t wanted = 0;
+    finfoobject* finfo;
+    apr_status_t result;
+
+    if (! PyArg_ParseTuple(args, "si", &fname, &wanted))
+        return NULL; 
+
+    finfo = (finfoobject *)MpFinfo_New();
+
+    fname = apr_pstrdup(finfo->pool, fname);
+
+    result = apr_stat(finfo->finfo, fname, wanted, finfo->pool);
+
+    if (result == APR_INCOMPLETE || result == APR_SUCCESS)
+        return (PyObject *)finfo;
+
+    if (result == APR_ENOENT)
+        return (PyObject *)finfo;
+
+    Py_DECREF(finfo);
+
+    PyErr_SetObject(PyExc_OSError,
+                    Py_BuildValue("is", result, "apr_stat() failed"));
+
+    return NULL;
+}
+
 /* methods of _apache */
 struct PyMethodDef _apache_module_methods[] = {
     {"config_tree",           (PyCFunction)config_tree,          METH_NOARGS},
@@ -692,6 +749,7 @@ struct PyMethodDef _apache_module_methods[] = {
     {"server_root",           (PyCFunction)server_root,          METH_NOARGS},
     {"register_cleanup",      (PyCFunction)register_cleanup,     METH_VARARGS},
     {"exists_config_define",  (PyCFunction)exists_config_define, METH_VARARGS},
+    {"stat",                  (PyCFunction)mp_stat,              METH_VARARGS},
     {"_global_lock",          (PyCFunction)_global_lock,         METH_VARARGS},
     {"_global_trylock",       (PyCFunction)_global_trylock,      METH_VARARGS},
     {"_global_unlock",        (PyCFunction)_global_unlock,       METH_VARARGS},
@@ -702,7 +760,7 @@ struct PyMethodDef _apache_module_methods[] = {
 
 DL_EXPORT(void) init_apache()
 {
-    PyObject *m, *d;
+    PyObject *m, *d, *o;
 
     /* initialize types XXX break windows? */
     MpTable_Type.ob_type = &PyType_Type; 
@@ -722,6 +780,43 @@ DL_EXPORT(void) init_apache()
 
     PyDict_SetItemString(d, "table", (PyObject *)&MpTable_Type);
 
+    o = PyInt_FromLong(AP_CONN_UNKNOWN);
+    PyDict_SetItemString(d, "AP_CONN_UNKNOWN", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(AP_CONN_CLOSE);
+    PyDict_SetItemString(d, "AP_CONN_CLOSE", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(AP_CONN_KEEPALIVE);
+    PyDict_SetItemString(d, "AP_CONN_KEEPALIVE", o);
+    Py_DECREF(o);
+
+    o = PyInt_FromLong(APR_NOFILE);
+    PyDict_SetItemString(d, "APR_NOFILE", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_REG);
+    PyDict_SetItemString(d, "APR_REG", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_DIR);
+    PyDict_SetItemString(d, "APR_DIR", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_CHR);
+    PyDict_SetItemString(d, "APR_CHR", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_BLK);
+    PyDict_SetItemString(d, "APR_BLK", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_PIPE);
+    PyDict_SetItemString(d, "APR_PIPE", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_LNK);
+    PyDict_SetItemString(d, "APR_LNK", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_SOCK);
+    PyDict_SetItemString(d, "APR_SOCK", o);
+    Py_DECREF(o);
+    o = PyInt_FromLong(APR_UNKFILE);
+    PyDict_SetItemString(d, "APR_UNKFILE", o);
+    Py_DECREF(o);
 }
 
 PyObject *get_ServerReturn() 
