@@ -15,7 +15,7 @@
  *
  * This file originally written by Stering Hughes
  * 
- * $Id: _pspmodule.c 102649 2004-02-16 19:47:28Z grisha $
+ * $Id: _pspmodule.c 431636 2006-08-15 16:22:19Z jgallacher $
  *
  * See accompanying documentation and source code comments for
  * details.
@@ -146,13 +146,18 @@ static PyObject * _psp_module_parsestring(PyObject *self, PyObject *argv)
     bs = yy_scan_string(PyString_AsString(str), scanner);
     yylex(scanner);
 
-    yy_delete_buffer(bs, scanner);
+    /* yy_delete_buffer(bs, scanner); */
     yylex_destroy(scanner);
     
     psp_string_0(&parser->pycode);
     Py_END_ALLOW_THREADS
 
-    code = PyString_FromString(parser->pycode.blob);
+    if (parser->pycode.blob) {
+        code = PyString_FromString(parser->pycode.blob);
+    }
+    else {
+        code = PyString_FromString("");
+    }
 
     psp_parser_cleanup(parser);
 
