@@ -18,7 +18,7 @@
  *
  * filterobject.c 
  *
- * $Id: filterobject.c 383365 2006-03-05 18:40:22Z jgallacher $
+ * $Id: filterobject.c 466105 2006-10-20 13:28:02Z jgallacher $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -75,7 +75,8 @@
 
 PyObject *MpFilter_FromFilter(ap_filter_t *f, apr_bucket_brigade *bb, int is_input,
                               ap_input_mode_t mode, apr_size_t readbytes,
-                              char * handler, char *dir)
+                              char *handler, PyObject *callable, char *dir,
+                              hl_entry *parent)
 {
     filterobject *result;
 
@@ -105,7 +106,9 @@ PyObject *MpFilter_FromFilter(ap_filter_t *f, apr_bucket_brigade *bb, int is_inp
     result->softspace = 0;
 
     result->handler = handler;
+    result->callable = callable;
     result->dir = dir;
+    result->parent = parent;
 
     result->request_obj = NULL; 
 
@@ -202,6 +205,7 @@ static PyObject *_filter_read(filterobject *self, PyObject *args, int readline)
     }
 
     bufsize = len < 0 ? HUGE_STRING_LEN : len;
+    /* PYTHON 2.5: 'PyString_FromStringAndSize' uses Py_ssize_t for input parameters */ 
     result = PyString_FromStringAndSize(NULL, bufsize);
 
     /* possibly no more memory */
@@ -257,6 +261,7 @@ static PyObject *_filter_read(filterobject *self, PyObject *args, int readline)
         /* time to grow destination string? */
         if (newline == 0 && len < 0 && bytes_read == bufsize) {
 
+            /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */
             _PyString_Resize(&result, bufsize + HUGE_STRING_LEN);
             buffer = PyString_AS_STRING((PyStringObject *) result);
             buffer += bytes_read;
@@ -296,6 +301,7 @@ static PyObject *_filter_read(filterobject *self, PyObject *args, int readline)
 
     /* resize if necessary */
     if (bytes_read < len || len < 0) 
+        /* PYTHON 2.5: '_PyString_Resize' uses Py_ssize_t for input parameters */
         if(_PyString_Resize(&result, bytes_read))
             return NULL;
 
@@ -352,7 +358,8 @@ static PyObject *filter_write(filterobject *self, PyObject *args)
         PyErr_SetString(PyExc_ValueError, "I/O operation on closed filter");
         return NULL;
     }
-
+    
+    /* PYTHON 2.5:  'PyString_Size' uses Py_ssize_t for return values (may need overflow check) */
     len = PyString_Size(s);
 
     if (len) {
@@ -492,7 +499,6 @@ static struct memberlist filter_memberlist[] = {
     {"name",               T_OBJECT,    0,                       RO},
     {"req",                T_OBJECT,    OFF(request_obj),          },
     {"is_input",           T_INT,       OFF(is_input),           RO},
-    {"handler",            T_STRING,    OFF(handler),            RO},
     {"dir",                T_STRING,    OFF(dir),                RO},
     {NULL}  /* Sentinel */
 };
@@ -548,6 +554,27 @@ static PyObject * filter_getattr(filterobject *self, char *name)
             return (PyObject *)self->request_obj;
         }
     }
+    else if (strcmp(name, "handler") == 0) {
+        if (self->callable) {
+            Py_INCREF(self->callable);
+            return self->callable;
+        } else if (self->handler) {
+            return PyString_FromString(self->handler);
+        }
+        else {
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
+    }
+    else if (strcmp(name, "parent") == 0) {
+        if (self->parent) {
+            return MpHList_FromHLEntry(self->parent);
+        }
+        else {
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
+    }
     else
         return PyMember_Get((char *)self, filter_memberlist, name);
 }
