@@ -18,7 +18,7 @@
  *
  * filterobject.h 
  *
- * $Id: filterobject.h 106619 2004-11-25 22:10:52Z nd $
+ * $Id: filterobject.h 426882 2006-07-30 10:27:00Z grahamd $
  *
  */
 
@@ -48,7 +48,9 @@ extern "C" {
         int bytes_written;
 
         char *handler;
+        PyObject *callable;
         char *dir;
+        hl_entry *parent;
 
         requestobject *request_obj;
 
@@ -61,7 +63,9 @@ extern "C" {
     extern DL_IMPORT(PyObject *) 
         MpFilter_FromFilter Py_PROTO((ap_filter_t *f, apr_bucket_brigade *bb_in, 
                                       int is_input, ap_input_mode_t mode, 
-                                      apr_size_t readbytes, char *hadler, char *dir));
+                                      apr_size_t readbytes, char *handler,
+                                      PyObject *callable, char *dir,
+                                      hl_entry *parent));
 
 #ifdef __cplusplus
 }
