@@ -18,7 +18,7 @@
  *
  * hlist.h 
  *
- * $Id: hlist.h 106619 2004-11-25 22:10:52Z nd $
+ * $Id: hlist.h 467228 2006-10-24 03:41:54Z grahamd $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -34,18 +34,31 @@ extern "C" {
     /* handler list entry */
     typedef struct hl_entry {
         const char *handler;
+        PyObject *callable;
         const char *directory;
+        int d_is_fnmatch;
+        ap_regex_t *d_regex;
+        const char *location;
+        int l_is_fnmatch;
+        ap_regex_t *l_regex;
         int silent;  /* 1 for PythonHandlerModule, where
                         if a handler is not found in a module,
                         no error should be reported */
         struct hl_entry *next;
+        struct hl_entry *parent;
     } hl_entry;
     
-    hl_entry *hlist_new(apr_pool_t *p, const char *h, const char *d, 
-                        const int s);
-    hl_entry *hlist_append(apr_pool_t *p, hl_entry *hle, const char * h,
-                           const char *d, const int s);
+    hl_entry *hlist_new(apr_pool_t *p, const char *h, PyObject* o,
+                        const char *d, int d_is_fnmatch, ap_regex_t *d_regex,
+                        const char *l, int l_is_fnmatch, ap_regex_t *l_regex,
+                        const int s, hl_entry* parent);
+    hl_entry *hlist_append(apr_pool_t *p, hl_entry *hle, const char * h, PyObject* o,
+                           const char *d, int d_is_fnmatch, ap_regex_t *d_regex,
+                           const char *l, int l_is_fnmatch, ap_regex_t *l_regex,
+                           const int s, hl_entry* parent);
+
     hl_entry *hlist_copy(apr_pool_t *p, const hl_entry *hle);
+    void hlist_extend(apr_pool_t *p, hl_entry *hle1, const hl_entry *hle2);
 
 #ifdef __cplusplus
 }
