@@ -18,7 +18,7 @@
  *
  * hlistobject.h 
  *
- * $Id: hlistobject.h,v 1.6 2004/02/16 19:47:28 grisha Exp $
+ * $Id: hlistobject.h 431327 2006-08-14 12:07:46Z grahamd $
  *
  * See accompanying documentation and source code comments 
  * for details.
@@ -34,16 +34,13 @@ extern "C" {
     typedef struct hlist {
         PyObject_HEAD
         struct hl_entry *head;
-        apr_pool_t *pool;
     } hlistobject;
 
     extern DL_IMPORT(PyTypeObject) MpHList_Type;
     
 #define MpHList_Check(op) ((op)->ob_type == &MpHList_Type)
 
-    extern DL_IMPORT(PyObject *)MpHList_FromHLEntry Py_PROTO((hl_entry *hse));
-    extern DL_IMPORT(void)MpHList_Append Py_PROTO((hlistobject *self, hl_entry *hse));
-    extern DL_IMPORT(void)MpHList_Append Py_PROTO((hlistobject *self, hl_entry *hle));
+    extern DL_IMPORT(PyObject *)MpHList_FromHLEntry Py_PROTO((hl_entry *hle));
     
 #ifdef __cplusplus
 }
