@@ -18,7 +18,7 @@
  *
  * tableobject.c 
  *
- * $Id: tableobject.c 106619 2004-11-25 22:10:52Z nd $
+ * $Id: tableobject.c 466105 2006-10-20 13:28:02Z jgallacher $
  *
  */
 
@@ -156,6 +156,7 @@ static int table_print(register tableobject *self, register FILE *fp, register i
 static PyObject * table_repr(tableobject *self)
 {
     PyObject *s;
+    PyObject *t = NULL;
     const apr_array_header_t *ah;
     apr_table_entry_t *elts;
     int i;
@@ -172,11 +173,22 @@ static PyObject * table_repr(tableobject *self)
     while (i--)
         if (elts[i].key)
         {
-            PyString_ConcatAndDel(&s, PyString_FromString("'"));
-            PyString_ConcatAndDel(&s, PyString_FromString(elts[i].key));
-            PyString_ConcatAndDel(&s, PyString_FromString("': '"));
-            PyString_ConcatAndDel(&s, PyString_FromString(elts[i].val));
-            PyString_ConcatAndDel(&s, PyString_FromString("'"));
+            t = PyString_FromString(elts[i].key);
+            PyString_ConcatAndDel(&s, PyObject_Repr(t));
+            Py_XDECREF(t);
+
+            PyString_ConcatAndDel(&s, PyString_FromString(": "));
+
+            if (elts[i].val) {
+              t = PyString_FromString(elts[i].val);
+            } else {
+              t = Py_None;
+              Py_INCREF(t);
+            }
+
+            PyString_ConcatAndDel(&s, PyObject_Repr(t));
+            Py_XDECREF(t);
+
             if (i > 0)
                 PyString_ConcatAndDel(&s, PyString_FromString(", "));
             else
@@ -224,6 +236,7 @@ static PyObject * table_subscript(tableobject *self, register PyObject *key)
        returns the first match.
     */
 
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */
     list = PyList_New(0);
     if (!list) 
         return NULL;
@@ -236,13 +249,21 @@ static PyObject * table_subscript(tableobject *self, register PyObject *key)
     while (i--)
         if (elts[i].key) {
             if (apr_strnatcasecmp(elts[i].key, k) == 0) {
-                PyObject *v = PyString_FromString(elts[i].val);
+                PyObject *v = NULL;
+                if (elts[i].val != NULL) {
+                    v = PyString_FromString(elts[i].val);
+                } else { 
+                    v = Py_None;
+                    Py_INCREF(v);
+                }
+                /* PYTHON 2.5: 'PyList_Insert' uses Py_ssize_t for input parameters */
                 PyList_Insert(list, 0, v);
                 Py_DECREF(v);
             }
         }
 
     /* if no match */
+    /* PYTHON 2.5: 'PyList_Size' uses Py_ssize_t for return values (may need overflow check) */
     if (PyList_Size(list) == 0) {
         Py_DECREF(list);
         PyErr_SetObject(PyExc_KeyError, key);
@@ -250,7 +271,9 @@ static PyObject * table_subscript(tableobject *self, register PyObject *key)
     }
 
     /* if we got one match */
+    /* PYTHON 2.5: 'PyList_Size' uses Py_ssize_t for return values (may need overflow check) */
     if (PyList_Size(list) == 1) {
+        /* PYTHON 2.5: 'PyList_GetItem' uses Py_ssize_t for input parameters */
         PyObject *v = PyList_GetItem(list, 0);
         Py_INCREF(v);
         Py_DECREF(list);
@@ -302,6 +325,7 @@ static int table_ass_subscript(tableobject *self, PyObject *key,
 /* table as mapping */
 
 static PyMappingMethods table_as_mapping = {
+    /* PYTHON 2.5: 'inquiry' should be perhaps replaced with 'lenfunc' */
     (inquiry)       tablelength,           /*mp_length*/
     (binaryfunc)    table_subscript,       /*mp_subscript*/
     (objobjargproc) table_ass_subscript,   /*mp_ass_subscript*/
@@ -325,6 +349,7 @@ static PyObject * table_keys(register tableobject *self)
     ah = apr_table_elts(self->table);
     elts = (apr_table_entry_t *) ah->elts;
 
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */
     v = PyList_New(ah->nelts);
 
     for (i = 0, j = 0; i < ah->nelts; i++)
@@ -332,6 +357,7 @@ static PyObject * table_keys(register tableobject *self)
         if (elts[i].key)
         {
             PyObject *key = PyString_FromString(elts[i].key);
+            /* PYTHON 2.5: 'PyList_SetItem' uses Py_ssize_t for input parameters */
             PyList_SetItem(v, j, key);
             j++;
         }
@@ -357,13 +383,21 @@ static PyObject * table_values(register tableobject *self)
     ah = apr_table_elts(self->table);
     elts = (apr_table_entry_t *) ah->elts;
 
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */
     v = PyList_New(ah->nelts);
 
     for (i = 0, j = 0; i < ah->nelts; i++)
     {
         if (elts[i].key)
         {
-            PyObject *val = PyString_FromString(elts[i].val);
+            PyObject *val = NULL;
+            if (elts[i].val != NULL) {
+                val = PyString_FromString(elts[i].val);
+            } else { 
+                val = Py_None;
+                Py_INCREF(val);
+            }
+            /* PYTHON 2.5: 'PyList_SetItem' uses Py_ssize_t for input parameters */
             PyList_SetItem(v, j, val);
             j++;
         }
@@ -389,6 +423,7 @@ static PyObject * table_items(register tableobject *self)
     ah = apr_table_elts(self->table);
     elts = (apr_table_entry_t *) ah->elts;
 
+    /* PYTHON 2.5: 'PyList_New' uses Py_ssize_t for input parameters */
     v = PyList_New(ah->nelts);
 
     for (i = 0, j = 0; i < ah->nelts; i++)
@@ -396,6 +431,7 @@ static PyObject * table_items(register tableobject *self)
         if (elts[i].key)
         {
             PyObject *keyval = Py_BuildValue("(s,s)", elts[i].key, elts[i].val);
+            /* PYTHON 2.5: 'PyList_SetItem' uses Py_ssize_t for input parameters */
             PyList_SetItem(v, j, keyval);
             j++;
         }
@@ -789,7 +825,15 @@ static int table_traverse(tableobject *self, visitproc visit, void *arg)
     while (i--)
         if (elts[i].key) {
             int err;
-            PyObject *v = PyString_FromString(elts[i].val);
+
+            PyObject *v = NULL;
+            if (elts[i].val != NULL) {
+                v = PyString_FromString(elts[i].val);
+            } else { 
+                v = Py_None;
+                Py_INCREF(v);
+            }
+
             err = visit(v, arg);
             Py_XDECREF(v);
             if (err)
@@ -842,7 +886,15 @@ static PyObject *select_key(apr_table_entry_t *elts)
 
 static PyObject *select_value(apr_table_entry_t *elts)
 {
-    return PyString_FromString(elts->val);
+    PyObject *val = NULL;
+    if (elts->val != NULL) {
+        val = PyString_FromString(elts->val);
+    } else { 
+        val = Py_None;
+        Py_INCREF(val);
+    }
+
+    return val;
 }
 
 static PyObject *select_item(apr_table_entry_t *elts)
@@ -946,11 +998,6 @@ static PySequenceMethods table_as_sequence = {
     0,                                  /* sq_inplace_repeat */
 };
 
-static PyObject *table_alloc(PyTypeObject *type, int nitems)
-{
-    return MpTable_New();
-}
-
 /**
  ** table_new
  **
@@ -958,20 +1005,7 @@ static PyObject *table_alloc(PyTypeObject *type, int nitems)
 
 static PyObject *table_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
-    PyObject *self;
-
-    assert(type != NULL && type->tp_alloc != NULL);
-    self = type->tp_alloc(type, 0);
-    if (self != NULL) {
-        apr_pool_t *p;
-        tableobject *t = (tableobject *)self;
-        apr_pool_create_ex(&p, NULL, NULL, NULL);
-        t->pool = p;
-        t->table = apr_table_make(p, 2);
-    }
-
-    return self;
-
+    return MpTable_New();
 }
 
 static int table_init(tableobject *self, PyObject *args, PyObject *kwds)
@@ -1038,6 +1072,7 @@ PyTypeObject MpTable_Type = {
     Py_TPFLAGS_BASETYPE,                /* tp_flags */
     mp_table_doc,                       /* tp_doc */
     (traverseproc)table_traverse,       /* tp_traverse */
+    /* PYTHON 2.5: 'inquiry' should be perhaps replaced with 'lenfunc' */
     (inquiry)table_tp_clear,            /* tp_clear */
     table_richcompare,                  /* tp_richcompare */
     0,                                  /* tp_weaklistoffset */
@@ -1052,7 +1087,7 @@ PyTypeObject MpTable_Type = {
     0,                                  /* tp_descr_set */
     0,                                  /* tp_dictoffset */
     (initproc)table_init,               /* tp_init */
-    (allocfunc)table_alloc,             /* tp_alloc */
+    0,                                  /* tp_alloc */
     table_new,                          /* tp_new */
     (destructor)table_dealloc,          /* tp_free */
 };
