@@ -1,58 +1,19 @@
- # ====================================================================
- # The Apache Software License, Version 1.1
  #
- # Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
- # reserved.
+ # 
+ # Licensed under the Apache License, Version 2.0 (the "License"); you
+ # may not use this file except in compliance with the License.  You
+ # may obtain a copy of the License at
  #
- # Redistribution and use in source and binary forms, with or without
- # modification, are permitted provided that the following conditions
- # are met:
+ #      http://www.apache.org/licenses/LICENSE-2.0
  #
- # 1. Redistributions of source code must retain the above copyright
- #    notice, this list of conditions and the following disclaimer.
+ # Unless required by applicable law or agreed to in writing, software
+ # distributed under the License is distributed on an "AS IS" BASIS,
+ # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+ # implied.  See the License for the specific language governing
+ # permissions and limitations under the License.
  #
- # 2. Redistributions in binary form must reproduce the above copyright
- #    notice, this list of conditions and the following disclaimer in
- #    the documentation and/or other materials provided with the
- #    distribution.
  #
- # 3. The end-user documentation included with the redistribution,
- #    if any, must include the following acknowledgment:
- #       "This product includes software developed by the
- #        Apache Software Foundation (http://www.apache.org/)."
- #    Alternately, this acknowledgment may appear in the software itself,
- #    if and wherever such third-party acknowledgments normally appear.
- #
- # 4. The names "Apache" and "Apache Software Foundation" must
- #    not be used to endorse or promote products derived from this
- #    software without prior written permission. For written
- #    permission, please contact apache@apache.org.
- #
- # 5. Products derived from this software may not be called "Apache",
- #    "mod_python", or "modpython", nor may these terms appear in their
- #    name, without prior written permission of the Apache Software
- #    Foundation.
- #
- # THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- # WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- # OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- # DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- # ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- # SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- # LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- # USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- # ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- # SUCH DAMAGE.
- # ====================================================================
- #
- # This software consists of voluntary contributions made by many
- # individuals on behalf of the Apache Software Foundation.  For more
- # information on the Apache Software Foundation, please see
- # <http://www.apache.org/>.
- #
- # $Id: tests.py 393776 2006-04-13 09:48:19Z grahamd $
+ # $Id: tests.py 474119 2006-11-13 03:04:44Z grahamd $
  #
 
 # mod_python tests
@@ -80,7 +41,10 @@ class SimpleTestCase(unittest.TestCase):
     def test_apache_log_error(self):
 
         s = self.req.server
+        c = self.req.connection
+
         apache.log_error("Testing apache.log_error():", apache.APLOG_INFO, s)
+
         apache.log_error("xEMERGx", apache.APLOG_EMERG, s)
         apache.log_error("xALERTx", apache.APLOG_ALERT, s)
         apache.log_error("xCRITx", apache.APLOG_CRIT, s)
@@ -90,6 +54,24 @@ class SimpleTestCase(unittest.TestCase):
         apache.log_error("xINFOx", apache.APLOG_INFO, s)
         apache.log_error("xDEBUGx", apache.APLOG_DEBUG, s)
 
+        s.log_error("xEMERGx", apache.APLOG_EMERG)
+        s.log_error("xALERTx", apache.APLOG_ALERT)
+        s.log_error("xCRITx", apache.APLOG_CRIT)
+        s.log_error("xERRx", apache.APLOG_ERR)
+        s.log_error("xWARNINGx", apache.APLOG_WARNING)
+        s.log_error("xNOTICEx", apache.APLOG_NOTICE)
+        s.log_error("xINFOx", apache.APLOG_INFO)
+        s.log_error("xDEBUGx", apache.APLOG_DEBUG)
+
+        c.log_error("xEMERGx", apache.APLOG_EMERG)
+        c.log_error("xALERTx", apache.APLOG_ALERT)
+        c.log_error("xCRITx", apache.APLOG_CRIT)
+        c.log_error("xERRx", apache.APLOG_ERR)
+        c.log_error("xWARNINGx", apache.APLOG_WARNING)
+        c.log_error("xNOTICEx", apache.APLOG_NOTICE)
+        c.log_error("xINFOx", apache.APLOG_INFO)
+        c.log_error("xDEBUGx", apache.APLOG_DEBUG)
+
         # see what's in the log now
         f = open("%s/logs/error_log" % apache.server_root())
         # for some reason re doesn't like \n, why?
@@ -97,7 +79,7 @@ class SimpleTestCase(unittest.TestCase):
         log = "".join(map(string.strip, f.readlines()))
         f.close()
 
-        if not re.search("xEMERGx.*xALERTx.*xCRITx.*xERRx.*xWARNINGx.*xNOTICEx.*xINFOx.*xDEBUGx", log):
+        if not re.search("xEMERGx.*xALERTx.*xCRITx.*xERRx.*xWARNINGx.*xNOTICEx.*xINFOx.*xDEBUGx.*xEMERGx.*xALERTx.*xCRITx.*xERRx.*xWARNINGx.*xNOTICEx.*xINFOx.*xDEBUGx.*xEMERGx.*xALERTx.*xCRITx.*xERRx.*xWARNINGx.*xNOTICEx.*xINFOx.*xDEBUGx", log):
             self.fail("Could not find test messages in error_log")
             
 
@@ -297,8 +279,10 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("req.phase should be 'PythonHandler'")
             
         log("    req.interpreter: %s" % `req.interpreter`)
+        if req.interpreter != apache.interpreter:
+            self.fail("req.interpreter should be same as apache.interpreter" % `apache.interpreter`)
         if req.interpreter != req.server.server_hostname:
-            self.fail("req.interpreter should be same as req.server_hostname: %s" % `req.server_hostname`)
+            self.fail("req.interpreter should be same as req.server.server_hostname: %s" % `req.server.server_hostname`)
             
         log("    req.content_type: %s" % `req.content_type`)
         log("        doing req.content_type = 'test/123' ...")
@@ -341,7 +325,7 @@ class SimpleTestCase(unittest.TestCase):
             
         log("    req.unparsed_uri: %s" % `req.unparsed_uri`)
         if req.unparsed_uri != "/tests.py":
-            self.fail("req.unparse_uri should be '/tests.py'")
+            self.fail("req.unparsed_uri should be '/tests.py'")
             
         log("    req.uri: %s" % `req.uri`)
         if req.uri != "/tests.py":
@@ -364,12 +348,12 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("req.args should be None")
             
         log("    req.finfo: %s" % `req.finfo`)
-        if req.finfo[10] and (req.finfo[10] != req.canonical_filename):
-            self.fail("req.finfo[10] should be the (canonical) filename")
+        if req.finfo[apache.FINFO_FNAME] and (req.finfo[apache.FINFO_FNAME] != req.canonical_filename):
+            self.fail("req.finfo[apache.FINFO_FNAME] should be the (canonical) filename")
         
         log("    req.parsed_uri: %s" % `req.parsed_uri`)
-        if req.parsed_uri[6] != '/tests.py':
-            self.fail("req.parsed_uri[6] should be '/tests.py'")
+        if req.parsed_uri[apache.URI_PATH] != '/tests.py':
+            self.fail("req.parsed_uri[apache.URI_PATH] should be '/tests.py'")
             
         log("    req.used_path_info: %s" % `req.used_path_info`)
         if req.used_path_info != 2:
@@ -389,6 +373,8 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("get_config return should show PythonDebug 1")
 
         log("req.get_options(): %s" % `req.get_options()`)
+        for option in apache.main_server.get_options().keys():
+            del req.get_options()[option]
         if req.get_options() != apache.table({"testing":"123"}):
             self.fail("get_options() should contain 'testing':'123', contains %s"%req.get_options().items())
 
@@ -495,6 +481,12 @@ class SimpleTestCase(unittest.TestCase):
         log = req.log_error
         conn = req.connection
 
+        try: 
+            import socket
+            localip = socket.gethostbyname("localhost") 
+        except: 
+            localip = "127.0.0.1"
+
         log("Examining connection memebers:")
 
         log("    connection.base_server: %s" % `conn.base_server`)
@@ -502,15 +494,15 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("conn.base_server should be same type as req.server")
         
         log("    connection.local_addr: %s" % `conn.local_addr`)
-        if not conn.local_addr[0] in ("127.0.0.1", "0.0.0.0"):
+        if not conn.local_addr[0] in ("127.0.0.1", "0.0.0.0", localip):
             self.fail("conn.local_addr[0] should be '127.0.0.1' or '0.0.0.0'")
         
         log("    connection.remote_addr: %s" % `conn.remote_addr`)
-        if not conn.remote_addr[0] in ("127.0.0.1", "0.0.0.0"):
+        if not conn.remote_addr[0] in ("127.0.0.1", "0.0.0.0", localip):
             self.fail("conn.remote_addr[0] should be '127.0.0.1' or '0.0.0.0'")
 
         log("    connection.remote_ip: %s" % `conn.remote_ip`)
-        if conn.remote_ip != "127.0.0.1":
+        if not conn.remote_ip in ("127.0.0.1", localip):
             self.fail("conn.remote_ip should be '127.0.0.1'")
 
         log("    connection.remote_host: %s" % `conn.remote_host`)
@@ -538,7 +530,7 @@ class SimpleTestCase(unittest.TestCase):
             self.fail("conn.keepalives should be 1")
 
         log("    connection.local_ip: %s" % `conn.local_ip`)
-        if conn.local_ip != "127.0.0.1":
+        if not conn.local_ip in ("127.0.0.1", localip):
             self.fail("conn.local_ip should be '127.0.0.1'")
 
         log("    connection.local_host: %s" % `conn.local_host`)
@@ -583,13 +575,6 @@ def handler(req):
 
     return apache.OK
 
-def req_add_handler(req):
-
-    req.secret_message = "foo"
-    req.add_handler("PythonHandler", "tests::simple_handler")
-
-    return apache.OK
-
 def simple_handler(req):
     # for req_add_handler()
     if (req.secret_message == "foo"):
@@ -597,6 +582,14 @@ def simple_handler(req):
         
     return apache.OK
 
+def req_add_handler(req):
+
+    req.secret_message = "foo"
+    req.add_handler("PythonHandler", "tests::simple_handler")
+    req.add_handler("PythonHandler", simple_handler)
+
+    return apache.OK
+
 def req_add_bad_handler(req):
     # bad_handler does not exist so adding it should 
     # should raise an AttributeError exception
@@ -617,6 +610,19 @@ def req_add_empty_handler_string(req):
 
     return apache.OK
 
+def req_add_handler_empty_phase(req):
+    req.log_error("req_add_handler_empty_phase")
+    req.log_error("phase=%s" % req.phase)
+    req.log_error("interpreter=%s" % req.interpreter)
+    req.log_error("directory=%s" % req.hlist.directory)
+    if req.phase != "PythonHandler":
+        directory = os.path.dirname(__file__)
+        req.add_handler("PythonHandler", "tests::req_add_handler_empty_phase", directory)
+    else:
+        req.write("test ok")
+
+    return apache.OK
+
 def accesshandler_add_handler_to_empty_hl(req):
     # Prior to version 3.2.6, adding a python handler 
     # to and empty handler list would cause a segfault
@@ -627,6 +633,32 @@ def accesshandler_add_handler_to_empty_hl(req):
 
     return apache.OK
 
+def test_req_add_handler_directory(req):
+    # dir1 will not have a trailing slash and on Win32
+    # will use back slashes and not forward slashes.
+    dir1 = os.path.dirname(__file__)
+    if req.phase == "PythonFixupHandler":
+        req.add_handler("PythonHandler", "tests::test_req_add_handler_directory", dir1)
+    else:
+	# dir2 should only use forward slashes and
+	# should have a trailing forward slash added by
+	# call to req.add_handler(). When dir1 and dir2
+	# are normalised for current operating system,
+        # they should be equivalent.
+        dir2 = req.hlist.directory
+        if dir2[-1] != '/' or dir2.count('\\') != 0:
+            req.write('test failed')
+        else:
+            dir1 = os.path.normpath(dir1)
+            dir2 = os.path.normpath(dir2)
+            if dir2 != dir1:
+                req.write('test failed')
+            else:
+                req.write('test ok')
+        
+    return apache.OK
+
+
 def req_allow_methods(req):
 
     req.allow_methods(["PYTHONIZE"])
@@ -634,14 +666,49 @@ def req_allow_methods(req):
 
 def req_get_basic_auth_pw(req):
 
-    if (req.phase == "PythonAuthenHandler"):
-        if req.user != "spam":
-            return apache.HTTP_UNAUTHORIZED
+    pw = req.get_basic_auth_pw()
+    if req.user != "spam" or pw != "eggs":
+        req.write("test failed")
     else:
         req.write("test ok")
 
     return apache.OK
 
+def req_auth_type(req):
+
+    if req.phase == "PythonAuthenHandler":
+        if req.auth_type() != "dummy":
+            req.log_error("auth_type check failed")
+            req.write("test failed")
+            return apache.DONE
+        if req.auth_name() != "blah":
+            req.log_error("auth_name check failed")
+            req.write("test failed")
+            return apache.DONE
+        req.user = "dummy"
+        req.ap_auth_type = req.auth_type()
+    elif req.phase == "PythonAuthzHandler":
+        if req.ap_auth_type != "dummy":
+            req.log_error("ap_auth_type check failed")
+            req.write("test failed")
+            return apache.DONE
+        if req.user != "dummy":
+            req.log_error("user check failed")
+            req.write("test failed")
+            return apache.DONE
+    else:
+        if req.ap_auth_type != "dummy":
+            req.log_error("ap_auth_type check failed")
+            req.write("test failed")
+            return apache.DONE
+        if req.user != "dummy":
+            req.log_error("user check failed")
+            req.write("test failed")
+            return apache.DONE
+        req.write("test ok")
+
+    return apache.OK
+
 def req_requires(req):
 
     if req.requires() == ('valid-user',):
@@ -670,6 +737,17 @@ def req_internal_redirect_int(req):
 
     return apache.OK
 
+def req_construct_url(req):
+
+    url = req.construct_url("/index.html")
+
+    if not re.match("^http://test_req_construct_url:[0-9]+/index.html$",url):
+        req.write("test failed")
+    else:
+        req.write("test ok")
+
+    return apache.OK
+
 def req_read(req):
 
     s = req.read()
@@ -688,14 +766,42 @@ def req_readline(req):
 
 def req_readlines(req):
 
-    lines = req.readlines()
+    
+    if 'SizeHint' in req.headers_in:
+        lines = req.readlines(int(req.headers_in['SizeHint']))
+    else:
+        lines = req.readlines()
+
     req.write("".join(lines))
 
     return apache.OK
 
+def req_discard_request_body(req):
+
+    s = req.read(10)
+    if s != '1234567890':
+        req.log_error('read() #1 returned %s' % `s`)
+        req.write('test failed')
+        return apache.OK
+
+    status = req.discard_request_body()
+    if status != apache.OK:
+        req.log_error('discard_request_body() returned %d' % status)
+        return status
+
+    s = req.read()
+    if s:
+        req.log_error('read() #2 returned %s' % `s`)
+        req.write('test failed')
+        return apache.OK
+
+    req.write('test ok')
+
+    return apache.OK
+
 def req_register_cleanup(req):
 
-    req.cleanup_data = "test ok"
+    req.cleanup_data = "req_register_cleanup test ok"
     req.register_cleanup(cleanup, req)
     req.write("registered cleanup that will write to log")
 
@@ -706,6 +812,11 @@ def cleanup(data):
 
     data.log_error(data.cleanup_data)
 
+def server_cleanup(data):
+    # for srv_register_cleanup and apache_register_cleanup below
+
+    apache.log_error(data)
+
 def req_headers_out(req):
 
     req.headers_out["X-Test-Header"] = "test ok"
@@ -761,9 +872,68 @@ def req_sendfile3(req):
     os.remove(fname)
     return apache.OK
 
+def req_handler(req):
+    if req.phase == "PythonFixupHandler":
+        req.handler = "mod_python"
+        req.handler = None
+        req.handler = "mod_python"
+        req.add_handler("PythonHandler","tests::req_handler")
+        return apache.OK
+    elif req.phase == "PythonHandler":
+        req.write('test ok')
+        return apache.OK
+    else:
+        req.write('test failed')
+        return apache.OK
+
+def req_no_cache(req):
+    req.no_cache = 1
+    req.write('test ok')
+    return apache.OK
+
+def req_update_mtime(req):
+    assert(req.mtime == 0.0)
+    req.update_mtime(100.0)
+    assert(req.mtime == 100.0)
+    req.set_etag()
+    req.set_last_modified()
+    req.write('test ok')
+    return apache.OK
+
+def util_redirect(req):
+    from mod_python import util
+    if req.main:
+        # Sub request for ErrorDocument.
+        req.write("test failed")
+        return apache.DONE
+    else:
+        if req.phase == "PythonFixupHandler":
+            util.redirect(req,location="/dummy",text="test ok")
+        else:
+            req.write('test failed')
+            return apache.OK
+
+def req_server_get_config(req):
+
+    if req.server.get_config().get("PythonDebug","0") != "1" or \
+            req.get_config().get("PythonDebug","0") != "0":
+        req.write('test failed')
+    else:
+        req.write('test ok')
+
+    return apache.OK
+
 def req_server_get_options(req):
 
     try:
+        server_options = apache.main_server.get_options()
+        assert(server_options.get("global","0") == "0")
+        assert(server_options.get("override","0") == "0")
+
+        server_options = req.connection.base_server.get_options()
+        assert(server_options.get("global","0") == "0")
+        assert(server_options.get("override","0") == "0")
+
         server_options = req.server.get_options()
         assert(server_options["global"] == "1")
         assert(server_options["override"] == "1")
@@ -790,16 +960,14 @@ def fileupload(req):
 
 def srv_register_cleanup(req):
 
-    req.cleanup_data = "test ok"
-    req.server.register_cleanup(req, cleanup, req)
+    req.server.register_cleanup(req, server_cleanup, "srv_register_cleanup test ok")
     req.write("registered server cleanup that will write to log")
 
     return apache.OK
 
 def apache_register_cleanup(req):
 
-    req.cleanup_data = "test 2 ok"
-    apache.register_cleanup(cleanup, req)
+    apache.register_cleanup(server_cleanup, "apache_register_cleanup test ok")
     req.write("registered server cleanup that will write to log")
 
     return apache.OK
@@ -819,6 +987,25 @@ def util_fieldstorage(req):
 
 def postreadrequest(req):
 
+    req.log_error('postreadrequest')
+
+    req.add_common_vars()
+
+    req.subprocess_env['TEST1'] = "'"
+    req.subprocess_env['TEST2'] = '"'
+
+    req.log_error('subprocess_env = %s' % req.subprocess_env)
+    req.log_error('subprocess_env.values() = %s' % req.subprocess_env.values())
+
+    for value in req.subprocess_env.itervalues():
+        req.log_error('VALUE = %s' % value)
+
+    for item in req.subprocess_env.iteritems():
+        req.log_error('ITEM = %s' % (item,))
+
+    req.log_error('SCRIPT_FILENAME = %s' % req.subprocess_env.get('SCRIPT_FILENAME'))
+    req.log_error('SCRIPT_FILENAME = %s' % req.subprocess_env['SCRIPT_FILENAME'])
+
     req.write("test ok")
 
     return apache.DONE
@@ -832,16 +1019,22 @@ def trans(req):
 
 def import_test(req):
 
-    import sys
+    import sys, os
+    directory = os.path.dirname(__file__)
+    assert(map(os.path.normpath, sys.path).count(directory) == 1)
     if sys.modules.has_key("dummymodule"):
-        req.write("test ok")
+        if not apache.main_server.get_options().has_key("dummymodule::function"):
+            req.log_error("dummymodule::function not executed")
+            req.write("test failed")
+        else:
+            req.write("test ok")
     else:
         req.log_error("dummymodule not found in sys.modules")
         req.write("test failed")
 
     return apache.OK
 
-def outputfilter(filter):
+def outputfilter1(filter):
 
     s = filter.read()
     while s:
@@ -853,8 +1046,49 @@ def outputfilter(filter):
 
     return apache.OK
 
+def outputfilter2(filter):
+
+    s = filter.read()
+    while s:
+        for c in s:
+          filter.write(2*c)
+        s = filter.read()
+
+    if s is None:
+        filter.close()
+
+    return apache.OK
+
 def simplehandler(req):
 
+    if req.phase != "PythonHandler":
+        req.write("test failed")
+        return apache.OK
+
+    req.write("test ok")
+
+    if req.phase != "PythonHandler":
+        req.write("test failed")
+        return apache.OK
+
+    return apache.OK
+
+def req_add_output_filter(req):
+
+    req.add_output_filter("MP_TEST_FILTER")
+
+    req.write("test ok")
+
+    return apache.OK
+
+def req_register_output_filter(req):
+
+    req.register_output_filter("MP_TEST_FILTER1","tests::outputfilter1")
+    req.register_output_filter("MP_TEST_FILTER2",outputfilter2)
+
+    req.add_output_filter("MP_TEST_FILTER1")
+    req.add_output_filter("MP_TEST_FILTER2")
+
     req.write("test ok")
 
     return apache.OK
@@ -936,6 +1170,66 @@ def Session_Session(req):
 
     return apache.OK
 
+def files_directive(req):
+
+    req.write(str(req.hlist.directory))
+    return apache.OK
+
+none_handler = None
+
+def server_return_1(req):
+    raise apache.SERVER_RETURN, apache.OK
+
+def server_return_2(req):
+    req.write("test ok")
+    return apache.OK
+
+def phase_status_1(req):
+    apache.log_error("phase_status_1")
+    req.phases = [1]
+    return apache.DECLINED
+
+def phase_status_2(req):
+    apache.log_error("phase_status_2")
+    req.phases.append(2)
+    req.user = "bogus"
+    req.ap_auth_type = "bogus"
+    return apache.OK
+
+def phase_status_3(req):
+    apache.log_error("phase_status_3")
+    req.phases.append(3)
+    return apache.OK
+
+def phase_status_4(req):
+    apache.log_error("phase_status_4")
+    #req.phases.append(4)
+    return apache.OK
+
+def phase_status_5(req):
+    apache.log_error("phase_status_5")
+    req.phases.append(5)
+    return apache.DECLINED
+
+def phase_status_6(req):
+    apache.log_error("phase_status_6")
+    req.phases.append(6)
+    return apache.OK
+
+def phase_status_7(req):
+    apache.log_error("phase_status_7")
+    req.phases.append(7)
+    return apache.OK
+
+def phase_status_8(req):
+    apache.log_error("phase_status_8")
+    apache.log_error("phases = %s" % req.phases)
+    if req.phases != [1, 2, 5, 6, 7]:
+        req.write("test failed")
+    else:
+        req.write("test ok")
+    return apache.OK
+
 def test_sys_argv(req):
     import sys
     req.write(repr(sys.argv))
@@ -954,8 +1248,14 @@ def PythonOption_items(req):
     return apache.OK
 
 def interpreter(req):
-    req.write(req.interpreter)
-    return apache.OK
+    if req.phase == "PythonFixupHandler":
+        if req.filename[-1] != '/' and os.path.isdir(req.filename):
+            req.write(req.interpreter)
+            return apache.DONE
+        return apache.OK
+    else:
+        req.write(req.interpreter)
+        return apache.DONE
 
 def index(req):
     return "test ok, interpreter=%s" % req.interpreter
@@ -963,6 +1263,33 @@ def index(req):
 def test_publisher(req):
     return "test ok, interpreter=%s" % req.interpreter
 
+def test_publisher_auth_nested(req):
+    def __auth__(req, user, password):
+        test_globals = test_publisher
+        req.notes["auth_called"] = "1"
+        return user == "spam" and password == "eggs"
+    def __access__(req, user):
+        req.notes["access_called"] = "1"
+        return 1
+    assert(int(req.notes.get("auth_called",0)))
+    assert(int(req.notes.get("access_called",0)))
+    return "test ok, interpreter=%s" % req.interpreter
+
+class _test_publisher_auth_method_nested:
+    def method(self, req):
+        def __auth__(req, user, password):
+            test_globals = test_publisher
+            req.notes["auth_called"] = "1"
+            return user == "spam" and password == "eggs"
+        def __access__(req, user):
+            req.notes["access_called"] = "1"
+            return 1
+        assert(int(req.notes.get("auth_called",0)))
+        assert(int(req.notes.get("access_called",0)))
+        return "test ok, interpreter=%s" % req.interpreter
+
+test_publisher_auth_method_nested = _test_publisher_auth_method_nested()
+
 class OldStyleClassTest:
     def __init__(self):
         pass
@@ -984,6 +1311,10 @@ def test_generator(req):
         yield c
         c += 1
 
+def server_side_include(req):
+    req.ssi_globals = { "data": "test" }
+    return apache.OK
+
 class InstanceTest(object):
     def __call__(self, req):
         return "test callable instance ok"
